     /**
     * \brief JSON Parser : Unit Tests
   *
   * \author Mihael Schmidt
   * \date   21.01.2011
   */

     H nomain

      *-------------------------------------------------------------------------
      * Prototypes
      *-------------------------------------------------------------------------
     D test_emptyObject...
     D                 PR
     D test_simpleObject...
     D                 PR
     D test_alternativeSyntax...
     D                 PR
     D test_nestedObject...
     D                 PR
     D test_simpleArray...
     D                 PR
     D test_nestedArray...
     D                 PR
     D test_emptyArrayWithoutObject...
     D                 PR
     D test_onlyArrays...
     D                 PR
     D test_arrayAndObject...
     D                 PR
     D test_unicodeCharacters...
     D                 PR
     D test_unicodeCharactersMixedCase...
     D                 PR
     
      /copy RPGUNIT1,TESTCASE
      /copy 'json/json_h.rpgle'
      /copy 'message_h.rpgle'


      *-------------------------------------------------------------------------
      * Procedures
      *-------------------------------------------------------------------------
 
     P test_emptyObject...
     P                 B                   export
      *
     D json            S               *
   /free
       json = json_util_fromFile('parsertest/test1.json' + x'00');
       assert(json <> *null : 'Parsed json object mustn''t be null.');
       iEqual(0 : json_size(json));
       
       json_dispose(json);
       
   /end-free
     P                 E

     P test_simpleObject...
     P                 B                   export
      *
     D json            S               *
      /free
       json = json_util_fromFile('parsertest/test2.json' + x'00');
       assert(json <> *null : 'Parsed json object mustn''t be null.');
       iEqual(1 : json_size(json));
       aEqual('value' : %str(json_getString(json : 'key')));
       
       json_dispose(json);
      /end-free
     P                 E
     
     P test_alternativeSyntax...
     P                 B                   export
      *
     D json            S               *
      /free
       json = json_util_fromFile('parsertest/test3.json' + x'00');
       assert(json <> *null : 'Parsed json object mustn''t be null.');
       iEqual(7 : json_size(json));
       aEqual('value1' : %str(json_getString(json : 'key1')));
       aEqual('value2' : %str(json_getString(json : 'key2')));
       aEqual('value3' : %str(json_getString(json : 'key3')));
       aEqual('value4' : %str(json_getString(json : 'key4')));
       assert(*on = json_getBoolean(json : 'special1') : 'Entry with key ' +
              'special1 should be true/*on.');
       assert(*off = json_getBoolean(json : 'special2') : 'Entry with key ' +
              'special2 should be false/*off.');
       assert(json_isNull(json : 'special3') : 'Entry with key ' +
              'special3 should be null.');
       
       
       json_dispose(json);
      /end-free
     P                 E
     
     
     P test_nestedObject...
     P                 B                   export
      *
     D json            S               *
     D nestedObject    S               *
      /free
       json = json_util_fromFile('parsertest/test4.json' + x'00');
       assert(json <> *null : 'Parsed json object mustn''t be null.');
       
       nestedObject = json_getObject(json : 'nested object');
       assert(nestedObject <> *null : 
              'One level nested json object mustn''t be null.');
       
       assert(json_contains(nestedObject : 'subsubobject') : 
              'Nested object should contain further nested json object ' + 
              'subsubobject.');
       
       nestedObject = json_getObject(nestedObject : 'subsubobject');
       assert(nestedObject <> *null : 
              '2nd level nested json object mustn''t be null.');
       
       nestedObject = json_getObject(nestedObject : 'value');
       assert(nestedObject <> *null : 
              '3rd level nested json object mustn''t be null.');
       assert(json_isEmpty(nestedObject) : '3rd level nested object should ' + 
                                           'be empty.');
       
       json_dispose(json);
      /end-free
     P                 E
     
     
     P test_simpleArray...
     P                 B                   export
      *
     D json            S               *
     D array           S               *
      /free
       json = json_util_fromFile('parsertest/test5.json' + x'00');
       assert(json <> *null : 'Parsed json object mustn''t be null.');
       
       assert(json_contains(json : 'product line ids') : 
              'JSON object should contain entry with key "product line ids".');
              
       array = json_getArray(json : 'product line ids');
       assert(array <> *null : 'JSON array should not be *null.');
       
       iEqual(4 : jsona_size(array));
       
       aEqual('200.1.1' : %str(jsona_getString(array : 0)));
       aEqual('200.1.2.1' : %str(jsona_getString(array : 1)));
       aEqual('200.10.10.1' : %str(jsona_getString(array : 2)));
       aEqual('200.11.11.1' : %str(jsona_getString(array : 3)));
       
       json_dispose(json);
      /end-free
     P                 E
     
     
     P test_nestedArray...
     P                 B                   export
      *
     D json            S               *
     D array           S               *
     D nestedArray     S               *
      /free
       json = json_util_fromFile('parsertest/test6.json' + x'00');
       assert(json <> *null : 'Parsed json object mustn''t be null.');
       
       assert(json_contains(json : 'product line ids') : 
              'JSON object should contain entry with key "product line ids".');
              
       array = json_getArray(json : 'product line ids');
       assert(array <> *null : 'JSON array should not be *null.');
       
       iEqual(4 : jsona_size(array));
       
       iEqual(JSON_TYPE_ARRAY : jsona_getEntryType(array : 0));
       nestedArray = jsona_getArray(array : 0);
       assert(nestedArray <> *null : 'Nested JSON array should not be *null.');
       iEqual(3 : jsona_size(nestedArray));
       iEqual(200 : jsona_getInt(nestedArray : 0));
       iEqual(1 : jsona_getInt(nestedArray : 1));
       iEqual(1 : jsona_getInt(nestedArray : 2));
       
       iEqual(JSON_TYPE_ARRAY : jsona_getEntryType(array : 1));
       nestedArray = jsona_getArray(array : 1);
       assert(nestedArray <> *null : 'Nested JSON array should not be *null.');
       iEqual(4 : jsona_size(nestedArray));
       iEqual(200 : jsona_getInt(nestedArray : 0));
       iEqual(10 : jsona_getInt(nestedArray : 1));
       iEqual(10 : jsona_getInt(nestedArray : 2));
       iEqual(1 : jsona_getInt(nestedArray : 3));
       
       iEqual(JSON_TYPE_ARRAY : jsona_getEntryType(array : 2));
       nestedArray = jsona_getArray(array : 2);
       assert(nestedArray <> *null : 'Nested JSON array should not be *null.');
       iEqual(4 : jsona_size(nestedArray));
       iEqual(200 : jsona_getInt(nestedArray : 0));
       iEqual(11 : jsona_getInt(nestedArray : 1));
       iEqual(11 : jsona_getInt(nestedArray : 2));
       iEqual(1 : jsona_getInt(nestedArray : 3));
       
       iEqual(JSON_TYPE_ARRAY : jsona_getEntryType(array : 3));
       nestedArray = jsona_getArray(array : 3);
       assert(nestedArray <> *null : 'Nested JSON array should not be *null.');
       iEqual(0 : jsona_size(nestedArray));
       
       json_dispose(json);
      /end-free
     P                 E
     
     
     P test_emptyArrayWithoutObject...
     P                 B                   export
      *
     D array           S               *
      /free
       array = json_util_fromFile('parsertest/test7.json' + x'00');
       assert(array <> *null : 'Parsed json array mustn''t be null.');
       
       iEqual(0 : jsona_size(array));
       
       jsona_dispose(array);
      /end-free
     P                 E
    
    
     P test_onlyArrays...
     P                 B                   export
      *
     D array           S               *
     D nestedArray     S               *
      /free
       array = json_util_fromFile('parsertest/test8.json' + x'00'); 
       assert(array <> *null : 'Parsed json array mustn''t be null.');
       
       iEqual(4 : jsona_size(array));
       
       iEqual(JSON_TYPE_ARRAY : jsona_getEntryType(array : 0));
       nestedArray = jsona_getArray(array : 0);
       assert(nestedArray <> *null : 'Nested JSON array should not be *null.');
       iEqual(3 : jsona_size(nestedArray));
       iEqual(200 : jsona_getInt(nestedArray : 0));
       iEqual(1 : jsona_getInt(nestedArray : 1));
       iEqual(1 : jsona_getInt(nestedArray : 2));
      
       iEqual(JSON_TYPE_ARRAY : jsona_getEntryType(array : 1));
       nestedArray = jsona_getArray(array : 1);
       assert(nestedArray <> *null : 'Nested JSON array should not be *null.');
       iEqual(4 : jsona_size(nestedArray));
       iEqual(200 : jsona_getInt(nestedArray : 0));
       iEqual(10 : jsona_getInt(nestedArray : 1));
       iEqual(10 : jsona_getInt(nestedArray : 2));
       iEqual(1 : jsona_getInt(nestedArray : 3));
       
       iEqual(JSON_TYPE_ARRAY : jsona_getEntryType(array : 2));
       nestedArray = jsona_getArray(array : 2);
       assert(nestedArray <> *null : 'Nested JSON array should not be *null.');
       iEqual(4 : jsona_size(nestedArray));
       iEqual(200 : jsona_getInt(nestedArray : 0));
       iEqual(11 : jsona_getInt(nestedArray : 1));
       iEqual(11 : jsona_getInt(nestedArray : 2));
       iEqual(1 : jsona_getInt(nestedArray : 3));
       
       iEqual(JSON_TYPE_ARRAY : jsona_getEntryType(array : 3));
       nestedArray = jsona_getArray(array : 3);
       assert(nestedArray <> *null : 'Nested JSON array should not be *null.');
       iEqual(0 : jsona_size(nestedArray));
        
       jsona_dispose(array);
      /end-free 
     P                 E
     
     P test_arrayAndObject...
     P                 B                   export
     D array           S               *
     D object          S               *
      /free
       array = json_util_fromFile('parsertest/test9.json' + x'00');
       assert(array <> *null : 'Parsed json array mustn''t be null.');
       
       iEqual(1 : jsona_size(array));
       
       iEqual(JSON_TYPE_OBJECT : jsona_getEntryType(array : 0));
       object = jsona_getObject(array : 0);
       iEqual(1 : json_size(object));
       iEqual(358 : json_getInt(object : 'id'));
       
       jsona_dispose(array);
      /end-free
     P                 E
     
     
     P test_unicodeCharacters...
     P                 B                   export
      *
     D json            S               *
     D umlaute_raw     S              3A
   /free
       umlaute_raw = u'00E400F600FC'; // u

       json = json_util_fromFile('parsertest/test10.json' + x'00');
       assert(json <> *null : 'Parsed json object mustn''t be null.');
       iEqual(3 : json_size(json));
       
       aEqual(umlaute_raw : %str(json_get(json : 'umlaute_raw')));
       
       json_dispose(json);
   /end-free
     P                 E
     
     
     P test_unicodeCharactersMixedCase...
     P                 B                   export
      *
     D json            S               *
     D umlaute_raw     S              3A
   /free
       umlaute_raw = u'00E400F600FC'; // u

       json = json_util_fromFile('parsertest/test11.json' + x'00');
       assert(json <> *null : 'Parsed json object mustn''t be null.');
       iEqual(3 : json_size(json));
       
       aEqual(umlaute_raw : %str(json_get(json : 'umlaute_raw')));
       
       json_dispose(json);
   /end-free
     P                 E
     